/*
 * libredBlocks_sim.h 
 * redBlocks Library - Version 2.13.0 - Evaluation Version.
 * Copyright  redlogix Software & System Engineering GmbH, 2012-2020. All rights reserved. 
 *
 * The source code in this file may be incomplete compared to its Commercial Version's counterpart. 
 * However, it may be used in its present form for developing commercial applications if at least 
 * one component of the redBlocks Library is commercially licensed for the respective application.
 * 
 * If not being used together with at least one commercially licensed component of the redBlocks 
 * Library it may be used for evaluation purposes ONLY. 
 *
 * In any case you may not: 
 * (a) to a third party, transfer rights or usage to, sublicense, rent, lease or otherwise distribute 
 *     the source code, or any portions thereof, except with redlogix's prior written consent.
 * (b) create derivative source code of the software.
 * (c) modify, loan, decompile, reverse engineer or disassemble or otherwise distribute the software, 
 *     except with redlogix's prior written consent.
 * (d) remove any intellectual property notices within the software.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITH POTENTIAL FAULTS.
 * NO WARRANTIES, WHETHER EXPLICITLY EXPRESSED, IMPLIED OR STATUTORY, INCLUDING, BUT
 * NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. REDLOGIX SHALL NOT, UNDER ANY
 * CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 */

#ifndef LIBREDBLOCKS_SIM_H_
#define LIBREDBLOCKS_SIM_H_

#include <rbCTypes.h>

#ifdef __cplusplus
extern "C"
{
#endif

/** Type definition of a generic callback function. */
typedef void ( *rb_sim_CallbackFunction )();

/** Enumeration that specifies for digital input interfaces, on which signal change the callback shall be invoked */
typedef enum
{
  RB_TRIGGER_EDGE_NONE = 0, RB_TRIGGER_EDGE_FALLING, RB_TRIGGER_EDGE_RISING, RB_TRIGGER_EDGE_BOTH
} rb_sim_DigitalInput_TriggerEdge;

/**
 * @ingroup module_target_sim_c_api
 * This function needs to be (or is automatically) called periodically from a different execution context.
 * From within this tick function, the redBlocks simulation library takes care of internal housekeeping tasks,
 * communicates with the simulator and invokes all driver stubs' callback functions. The context that periodically
 * calls this function is the simulation context.
 *
 * On platforms without an operating system, this function is typically called from the system tick or a timer interrupt service routine.
 * On platforms with an operating system, this function is commonly called from within an extra thread.
 *
 * Depending on the simulation environment's setup, either the user is responsible to write code that invokes this function periodically.
 * Or the simulation environment automatically sets up a simulation context that calls this function periodically. Refer to the documentation
 * of the simulation environment's init function (e. g. rb_sim_posix_initTcp()) in order to find out whether you need to call this function
 * periodically or not.
 */
void rb_sim_tick( void );

/**
 * @ingroup module_target_sim_c_api
 * This method simulates entering the processor's sleep mode.
 * It does not return before leaveSleepMode() is called.
 */
void rb_sim_enterSleepMode( void );

/**
 * @ingroup module_target_sim_c_api
 * When this method is called, a previous call (from a different execution context) to enterSleepMode() returns.
 */
void rb_sim_leaveSleepMode( void );

/**
 * @ingroup module_target_sim_c_api
 * This method needs to be called after the simulated hardware has been initialized.
 * It sends a message to the simulation server indicating that we have reached this point.
 * It will tell the simulator that all interfaces have been initialized and the simulation state
 * can be changed from @c started to @c running.
 */
void rb_sim_initDone( void );

/*******************************************************************************************
 * System tick
 *******************************************************************************************/

/**
 * @ingroup module_target_sim_c_api
 * With this function the system tick simulation can be set up. The specified function arguments
 * define how often the callback function is invoked within the specified number of simulation ticks.
 * Each call to the function @ref rb_sim_tick is a simulation tick.
 *
 * Example: numSysTicks=10 and perSimTicks=1 => on each call to @ref rb_sim_tick, 10 system ticks will
 * be simulated, i. e. the specified callback function will be invoked 10 times from within each call to
 * @ref rb_sim_tick.
 *
 * The function does not yet enable the system tick. This must be done by calling @ref rb_sim_SystemTick_enableCallback().
 * @param numSysTicks the number of system ticks that will be simulated when the method
 *   @ref rb_sim_tick has been called @par perSimTick times.
 * @param perSimTicks the number of simulation ticks that need to pass until the specified number
 *   of @par numSysTicks are simulated.
 * @param callback the callback function which is invoked on each simulated system tick
 */
void rb_sim_SystemTick_init( u8 numSysTicks, u8 perSimTicks, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * Enables the simulated system tick.
 */
void rb_sim_SystemTick_enableCallback( void );

/**
 * @ingroup module_target_sim_c_api
 * Disables the simulated system tick.
 */
void rb_sim_SystemTick_disableCallback( void );

/**
 * @ingroup module_target_sim_c_api
 * Use this function to temporarily disable all simulated interrupts. After having called this function,
 * no driver stub will issue any more callbacks (but callback requests will be queued) until they are
 * re-enabled by a call to rb_sim_enableAllCallbacks().
 *
 * Be aware that this function also deactivates the simulation tick, which is required to keep up
 * the communication with the redBlocks Simulator. Thus, make sure to disable the interrupts only
 * temporarily.
 *
 * When the simulation environment starts, the callbacks are automatically enabled.
 */
void rb_sim_disableAllCallbacks( void );

/**
 * @ingroup module_target_sim_c_api
 * Use this function to re-enable the simulated interrupts that have been disabled by a call to
 * rb_sim_disableAllCallbacks().
 */
void rb_sim_enableAllCallbacks( void );

/*******************************************************************************************
 * Interfaces
 * Before interfaces can be used, they must be created with the corresponding add function.
 * Within the add functions, an instance of the interface is created and the initial value is exchanged with the simulator.
 * Before calling an add method make sure the following preconditions have been met.
 * - The function rb_sim_tick() needs to be called periodically.
 *   Some initialization functions like rb_sim_posix_initTcp() internally creates a tick source and call the function rb_sim_tick().
 *   In other cases, the tick source needs to be set up manually, i.e. start a hardware timer and call
 *   the function rb_sim_tick() from the interrupt service routine.
 * - The connection to the simulator needs to be established.
 *******************************************************************************************/

/**
 * @ingroup module_target_sim_c_api
 * Adds a new digital input interface.
 * This function blocks until the initial value of the simulated peripheral was received from the simulator.
 * @param index the unique interface identifier.
 * @param triggerEdge specifies at which signal edge the callback is triggered.
 * @param callback Pointer to the callback function which is invoked each time the interface changes (depending on the configured triggerEdge).
 *                 Callback invocation is deactivated by specifying NULL.
 *                 Callbacks are disabled by default and need to be enabled manually.
 *                 Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_DigitalInput_add( u16 index, rb_sim_DigitalInput_TriggerEdge triggerEdge, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * Adds a new digital output interface.
 * @param index the unique interface identifier.
 * @param initialValue use this value as initial value.
 */
void rb_sim_DigitalOutput_add( u16 index, bool initialValue );

/**
 * @ingroup module_target_sim_c_api
 * Adds a new analog input interface.
 * This function blocks until the initial value of the simulated peripheral was received from the simulator.
 * @param index the unique interface identifier.
 * @param maxValue specifies the maximum value of the interface, the minimum value is always 0.
 * @param callback Pointer to the callback function which is invoked each time the interface changes.
 *                 Callback invocation is deactivated by specifying NULL.
 *                 Callbacks are disabled by default and need to be enabled manually.
 *                 Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_AnalogInput_add( u16 index, u32 maxValue, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * Adds a new analog output interface.
 * @param index the unique interface identifier.
 * @param initialValue use this value as initial value.
 * @param maxValue specifies the maximum value of the interface, the minimum value is always 0.
 */
void rb_sim_AnalogOutput_add( u16 index, u32 initialValue, u32 maxValue );

/**
 * @ingroup module_target_sim_c_api
 * Adds a new touch input interface.
 * This function blocks until the initial touch input state was received from the simulator.
 * @param index the unique interface identifier.
 * @param callback Pointer to the callback function which is invoked each time the touch input state changes.
 *                 Callback invocation is deactivated by specifying NULL.
 *                 Callbacks are disabled by default and need to be enabled manually.
 *                 Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_TouchInput_add( u16 index, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * Adds a new communication channel interface.
 * ComChannels are a generic way to exchange data with the simulator, whereas ComInterfaces are specialized
 * for simulating communication interfaces (e. g. UART, SPI, parallel data interface, ...).
 * ComChannels work packet-oriented, i. e. they transfer data as distinct messages.
 * @note Be aware that ComChannel- and ComInterfaces share the same range of indices.
 * @param index the unique interface identifier.
 * @param sendQueueSize the size of the internal send queue that is needed to buffer the outgoing data.
 *                 Be aware that the maximum size of a data message that can be sent to the simulator is sendQueueSize-1.
 * @param recvQueueSize the size of the internal receive queue that is needed to buffer the incoming data.
 *                 Be aware that data messages with a size > recvQueueSize-1 will be split up by the simulator in multiple messages. Smaller messages
 *                 will never be split up.
 * @param callback Pointer to the callback function which is invoked each time a new message was received.
 *                 Callback invocation is deactivated by specifying NULL.
 *                 Callbacks are disabled by default and need to be enabled manually.
 *                 Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_ComChannel_add( u16 index, u8 sendQueueSize, u8 recvQueueSize, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * Adds a new communication interface.
 * While ComChannels are a generic way to exchange data with the simulator, ComInterfaces are used to
 * simulate communication interfaces (e. g. UART, parallel data interface, ...).
 * ComInterfaces work stream-oriented (i. e. the data may be received in different portions than they were sent).
 * @note Be aware that ComChannel- and ComInterfaces share the same range of indices.
 * @param index the unique interface identifier.
 * @param sendFifoSize the size of the send FIFO of the simulated device, i. e. portions of this size can be sent when being notified by a send callback.
 * @param recvFifoSize the size of the receive FIFO of the simulated device, i. e. at most that many bytes will be available after being notified by a receive callback.
 * @param sendQueueSize the size of the internal send queue that is needed to buffer the outgoing data (should be >= sendFifoSize).
 * @param recvQueueSize the size of the internal receive queue that is needed to buffer the incoming data (should be >= recvFifoSize).
 * @param sendCallback Pointer to the callback function which is invoked each time data can be sent or NULL in order to switch off send callback invocation.
 *                     Callbacks are disabled by default and need to be enabled manually.
 *                     Be aware that the callback is invoked from the simulation tick context.
 * @param recvCallback Pointer to the callback function which is invoked each time data was received or NULL in order to switch off receive callback invocation.
 *                     Callbacks are disabled by default and need to be enabled manually.
 *                     Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_ComInterface_add( u16 index, u8 sendFifoSize, u8 recvFifoSize, u8 sendQueueSize, u8 recvQueueSize, rb_sim_CallbackFunction sendCallback, rb_sim_CallbackFunction recvCallback );

/**
 * @ingroup module_target_sim_c_api
 * Adds a new display interface.
 * @param index the unique interface identifier.
 * @param width the number of horizontal pixels.
 *              The specified width must be a multiple of 8.
 * @param height the number of vertical pixels.
 *               The specified height must be a multiple of 8.
 * @param bitsPerPixel the amount of bits, used for representing the pixel color.
 *                     While the driver supports arbitrary values, the simulator is currently restricted to the values 1, 2, 4 and 8.
 * @param autoUpdate specifies if an update should be automatically scheduled after writing to the display.
 *                   If false is given, the update needs to be triggered manually by calling the function rb_sim_Display_update().
 */
void rb_sim_Display_add( u16 index, u16 width, u16 height, u8 bitsPerPixel, bool autoUpdate );

/**
 * @ingroup module_target_sim_c_api
 * Adds a new memory interface (ROM, RAM and NVRAM).
 * @note Be aware that all memory interfaces share the same range of indices (independently from the type of memory).
 * @param index the unique interface identifier.
 * @param memoryAddress the start address of the simulated memory.
 * @param memorySize the size of the simulated memory.
 * @param callback the callback function which is invoked after the last operation has been finished or NULL.
 *                 Callbacks are disabled by default and need to be enabled manually.
 *                 Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_Memory_add( u16 index, u32 memoryAddress, u32 memorySize, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * Adds a new flash memory interface.
 * @note Be aware that Memory- and FlashMemory interfaces share the same range of indices (independently from the type of memory).
 * @param index the unique interface identifier.
 * @param memoryAddress the start address of the simulated memory.
 * @param memorySize the size of the simulated memory.
 *                   The specified size needs to be a multiple of the specified pageSize.
 * @param pageSize the size of each flash page.
 * @param erasedMemoryValue the byte value of the memory after an erase operation (e. g. 0xFF).
 * @param callback the callback function which is invoked after the last operation (i. e. read, write, erase) is finished or NULL.
 *                 Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_Memory_Flash_add( u16 index, u32 memoryAddress, u32 memorySize, u32 pageSize, u8 erasedMemoryValue, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * Updates the value of a digital output interface.
 * @param index the unique interface identifier.
 * @param value the new value.
 */
void rb_sim_DigitalOutput_setValue( u16 index, bool value );

/**
 * @ingroup module_target_sim_c_api
 * Returns the current value of a digital output interface.
 * @param index the unique interface identifier.
 * @return the value of the output interface.
 */
bool rb_sim_DigitalOutput_getValue( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Returns the value of a digital input interface.
 * @param index the unique interface identifier.
 * @return the value of the interface.
 */
bool rb_sim_DigitalInput_getValue( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Updates the callback function of a digital input interface.
 * @param index the unique interface identifier.
 * @param callback Pointer to the callback function (or NULL in order to unset a previously set callback function) which is invoked each time the signal edge changes, according to the configured triggerEdge.
 *                 Callbacks are disabled by default and need to be enabled manually.
 *                 Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_DigitalInput_setCallback( u16 index, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * Updates the trigger edge of a digital input interface.
 * @param index the unique interface identifier.
 * @param triggerEdge specifies at which signal edge the callback is triggered.
 */
void rb_sim_DigitalInput_setTriggerEdge( u16 index, rb_sim_DigitalInput_TriggerEdge triggerEdge );

/**
 * @ingroup module_target_sim_c_api
 * (Re-)Enables the callback invocation of a digital input interface.
 * @param index the unique interface identifier.
 */
void rb_sim_DigitalInput_enableCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Disables the callback invocation of a digital input interface.
 * @param index the unique interface identifier.
 */
void rb_sim_DigitalInput_disableCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Updates the value of an analog output interface.
 * @param index the unique interface identifier.
 * @param value the new value. If the value exceeds the configured maximum value it is limited to this maximum value.
 */
void rb_sim_AnalogOutput_setValue( u16 index, u32 value );

/**
 * @ingroup module_target_sim_c_api
 * Returns the value of an analog input interface.
 * @param index the unique interface identifier.
 * @return the value of the interface.
 */
u32 rb_sim_AnalogInput_getValue( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Updates the callback function of an analog input interface.
 * @param index the unique interface identifier.
 * @param callback pointer to the callback function (or NULL in order to unset a previously set callback function) which is invoked each time a sampling result is available.
 *                 Callbacks are disabled by default and need to be enabled manually.
 *                 Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_AnalogInput_setCallback( u16 index, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * (Re-)Enables the callback invocation of an analog input interface.
 * @param index the unique interface identifier.
 */
void rb_sim_AnalogInput_enableCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Disables the callback invocation of an analog input interface.
 * @param index the unique interface identifier.
 */
void rb_sim_AnalogInput_disableCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Starts sampling of an analog input interface.
 * The callback will be invoked after sampling is finished.
 * @param index the unique interface identifier.
 */
void rb_sim_AnalogInput_sampleOnce( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Starts continuous sampling of an analog input interface.
 * The callback will be invoked at the end of each sampling step.
 * @param index the unique interface identifier.
 */
void rb_sim_AnalogInput_sampleContinuousStart( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Stops continuous sampling of an analog input interface.
 * @param index the unique interface identifier.
 */
void rb_sim_AnalogInput_sampleContinuousStop( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Returns the coordinates of the last pressed or unpressed position.
 * @param index the unique interface identifier.
 * @param x pointer to a variable where the x coordinate is stored.
 * @param y pointer to a variable where the y coordinate is stored.
 * @return this value can be false (unpressed) or true (pressed).
 */
bool rb_sim_TouchInput_getState( u16 index, u16* x, u16* y );

/**
 * @ingroup module_target_sim_c_api
 * Updates the callback function of a touch input interface.
 * @param index the unique interface identifier.
 * @param callback Pointer to the callback function (or NULL in order to unset a previously set callback function) which is invoked each time the state of the interface changes.
 *                 Callbacks are disabled by default and need to be enabled manually.
 *                 Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_TouchInput_setCallback( u16 index, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * (Re-)Enables the callback invocation of a touch input interface.
 * @param index the unique interface identifier.
 */
void rb_sim_TouchInput_enableCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Disables the callback invocation of a touch input interface.
 * @param index the unique interface identifier.
 */
void rb_sim_TouchInput_disableCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Sends a message using a communication channel.
 * @attention This function must not be called concurrently for the same communication channel, otherwise the program may crash!
 * @note  Be aware that the maximum size of a data message that can be sent to the simulator is the sendQueueSize-1
 *        that is specified during initialization with a call to @ref rb_sim_ComChannel_add.
 * @param index the unique interface identifier.
 * @param data pointer to the data to send.
 * @param dataLen length of the data to send.
 * @return true if the complete message was successfully sent, false if there is not enough space available (in this case, no data is sent).
 */
bool rb_sim_ComChannel_sendMessage( u16 index, const void* data, u8 dataLen );

/**
 * @ingroup module_target_sim_c_api
 * Moves the internal read index to the next received communication channel message.
 * This function must be called before any data bytes of the received message can be read.
 * @attention Retrieving the next receive message and reading from it must not done concurrently for
 *   the same communication channel, otherwise the program may crash!
 * @param index the unique interface identifier.
 * @return the size of the received message, or 0 if no (other) message was received
 */
u8 rb_sim_ComChannel_getNextRecvMessage( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Returns the data of a communication channel message.
 * Be aware that the function rb_sim_ComChannel_getNextRecvMessage must have been called before.
 * The received message data can be read in chunks by calling this function multiple times and
 * passing a value < the received message length to the parameter @c dataLen.
 * @attention Retrieving the next receive message and reading from it must not done concurrently for
 *   the same communication channel, otherwise the program may crash!
 * @param index the unique interface identifier.
 * @param data pointer to a buffer where the received data is copied to.
 * @param dataLen size of the given buffer.
 * @return Number of bytes that are actually copied. If the returned value is < @c dataLen, no
 *         more message bytes are available and further calls to this function (without calling
 *         rb_sim_ComChannel_getNextRecvMessage in between) will return 0.
 */
u8 rb_sim_ComChannel_getRecvMessageData( u16 index, void* data, u8 dataLen );

/**
 * @ingroup module_target_sim_c_api
 * Updates the received data callback function of a communication channel.
 * @param index the unique interface identifier.
 * @param callback pointer to the callback function (or NULL in order to unset a previously set callback function) which will be invoked each time a new message has been received.
 *                 Callbacks are disabled by default and need to be enabled manually.
 *                 Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_ComChannel_setRecvDataCallback( u16 index, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * (Re-)Enables the received data callback invocation of a communication channel.
 * @param index the unique interface identifier.
 */
void rb_sim_ComChannel_enableCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Disables the received data callback invocation of a communication channel.
 * @param index the unique interface identifier.
 */
void rb_sim_ComChannel_disableCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Returns the received data of a communication interface.
 * @param index the unique interface identifier.
 * @param data pointer to a buffer where the received data is copied to.
 * @param dataLen size of the given buffer.
 * @return number of bytes that are copied.
 */
u8 rb_sim_ComInterface_getRecvData( u16 index, void* data, u8 dataLen );

/**
 * @ingroup module_target_sim_c_api
 * Sends data using a communication interface.
 * @param index the unique interface identifier.
 * @param data pointer to the data to send.
 * @param dataLen length of the data to send.
 * @return the number of bytes that could be queued.
 */
u8 rb_sim_ComInterface_putSendData( u16 index, const void* data, u8 dataLen );

/**
 * @ingroup module_target_sim_c_api
 * Indicates whether data is available to be retrieved from a communication interface.
 * @param index the unique interface identifier.
 * @return true if data is available.
 */
bool rb_sim_ComInterface_getIsRecvDataAvailable( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Indicates whether data can be put into the send FIFO of a communication interface.
 * @param index the unique interface identifier.
 * @return true, if there is space free in the send FIFO.
 */
bool rb_sim_ComInterface_getIsSendSpaceAvailable( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Updates the received data callback of a communication interface.
 * @param index the unique interface identifier.
 * @param callback pointer to the callback function (or NULL in order to unset a previously set callback function) which is invoked each time new data has been received.
 *                 Callbacks are disabled by default and need to be enabled manually.
 *                 Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_ComInterface_setRecvDataCallback( u16 index, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * Updates the send ready callback of a communication interface.
 * @param index the unique interface identifier.
 * @param callback pointer to the callback function (or NULL in order to unset a previously set callback function) which is invoked each time data can be send.
 *                 Callbacks are disabled by default and need to be enabled manually.
 *                 Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_ComInterface_setSendReadyCallback( u16 index, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * (Re-)Enables the send ready callback invocation of a communication interface.
 * @param index the unique interface identifier.
 */
void rb_sim_ComInterface_enableSendReadyCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Disables the send ready callback invocation of a communication interface.
 * @param index the unique interface identifier.
 */
void rb_sim_ComInterface_disableSendReadyCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * (Re-)Enables the receive data callback invocation of a communication interface.
 * @param index the unique interface identifier.
 */
void rb_sim_ComInterface_enableRecvDataCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Disables the received data callback invocation of a communication interface.
 * @param index the unique interface identifier.
 */
void rb_sim_ComInterface_disableRecvDataCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Returns the width of a display interface.
 * @param index the unique interface identifier.
 * @return the width of the display in pixel.
 */
u16 rb_sim_Display_getWidth( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Returns the height of a display interface.
 * @param index the unique interface identifier.
 * @return the height of the display in pixel.
 */
u16 rb_sim_Display_getHeight( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Updates the color value of a display pixel. The change will not be reflected in the
 * simulator before calling rb_sim_Display_update().
 * @param index the unique interface identifier.
 * @param x the X-Coordinate of the pixel, starting from 0.
 * @param y the Y-Coordinate of the pixel, starting from 0.
 * @param value the new pixels color value.
 */
void rb_sim_Display_setPixel( u16 index, u16 x, u16 y, u32 value );

/**
 * @ingroup module_target_sim_c_api
 * Returns the color value of a display pixel.
 * @param index the unique interface identifier.
 * @param x the X-Coordinate of the pixel, starting from 0.
 * @param y the Y-Coordinate of the pixel, starting from 0.
 * @return the pixel's color value.
 */
u32 rb_sim_Display_getPixel( u16 index, u16 x, u16 y );

/**
 * @ingroup module_target_sim_c_api
 * Transfers the frame buffer of a display to the simulator. This function must be called,
 * after one or more pixels haven been updated.
 * @param index the unique interface identifier.
 */
void rb_sim_Display_update( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Reads data from a memory interface.
 * This is an asynchronous function, which means the operation will be started and the function returns immediately.
 * A callback will be invoked after the operation has been finished.
 * Before this operation is finished no other operation of any kind (read, write, erase) must be started for
 * the simulated memory with the same index.
 * @param index the unique interface identifier.
 * @param address the address of the data region to read.
 * @param data pointer to a buffer where the data is copied to.
 *             Make sure the specified buffer stays valid until the operation has been finished.
 * @param dataLen size of the given buffer, i.e. number of bytes to read.
 */
void rb_sim_Memory_read( u16 index, u32 address, void* data, u32 dataLen );

/**
 * @ingroup module_target_sim_c_api
 * Writes data to a memory interface.
 * This is an asynchronous function, which means the operation will be started and the function returns immediately.
 * A callback will be invoked after the operation has been finished.
 * Before this operation is finished no other operation of any kind (read, write, erase) must be started for
 * the simulated memory with the same index.
 * @param index the unique interface identifier.
 * @param address the address of the data region where the data is written to.
 * @param data pointer to the data that needs to be written.
 *             Make sure the specified data stays valid until the operation has been finished.
 * @param dataLen size of the given data, i.e. number of bytes to write.
 */
void rb_sim_Memory_write( u16 index, u32 address, const void* data, u32 dataLen );

/**
 * @ingroup module_target_sim_c_api
 * Erases a flash page of a flash memory interface.
 * This is an asynchronous function, which means the operation will be started and the function returns immediately.
 * A callback will be invoked after the operation has been finished.
 * Before this operation is finished no other operation of any kind (read, write, erase) must be started for
 * the simulated memory with the same index.
 * @param index the unique interface identifier.
 * @param address the start address of the flash page to erase.
 */
void rb_sim_Memory_erasePage( u16 index, u32 address );

/**
 * @ingroup module_target_sim_c_api
 * Updates the operation finished callback of a memory interface.
 * @param index the unique interface identifier.
 * @param callback pointer to the callback function (or NULL in order to unset a previously set callback function) which is invoked after the last operation has been finished.
 *                 Callbacks are disabled by default and need to be enabled manually.
 *                 Be aware that the callback is invoked from the simulation tick context.
 */
void rb_sim_Memory_setCallback( u16 index, rb_sim_CallbackFunction callback );

/**
 * @ingroup module_target_sim_c_api
 * (Re-)Enables the operation finished callback invocation of a memory interface.
 * @param index the unique interface identifier.
 */
void rb_sim_Memory_enableCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Disables the operation finished callback invocation of a memory interface.
 * @param index the unique interface identifier.
 */
void rb_sim_Memory_disableCallback( u16 index );

/**
 * @ingroup module_target_sim_c_api
 * Returns the flash page size of a flash memory interface.
 * @param index the unique interface identifier.
 * @param address the start address of the flash page.
 *                The given address needs to be the exact start address, otherwise 0 is returned.
 * @return size of the flash page, 0 if given address is not a valid flash page start address.
 */
u32 rb_sim_Memory_getPageSize( u16 index, u32 address );

#ifdef __cplusplus
}
#endif

#endif // LIBREDBLOCKS_SIM_H_
